/*+
|                                                   |
|          Procedure: ETL_ORDPRODUCAO               |
|                                                   |
|                                                   |
|     Criao:                                      |
|     01/08/2011 - Kenia Soares                     |
|                                                   |
|                                                   |
+*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE
	        O.NAME = 'ETL_ORDPRODUCAO'  AND
	        TYPE = 'P' 		    AND
	        U.UID = O.UID               AND
	        U.NAME = USER)

DROP PROCEDURE ETL_ORDPRODUCAO
GO

create PROCEDURE ETL_ORDPRODUCAO
                     (@P_DTINI      DATETIME,
                      @P_DTFINA     DATETIME)
AS
   /* DATA ATUAL */
   DECLARE @V_MESINI          VARCHAR(2); /*Ms*/
   DECLARE @V_ANOINI          VARCHAR(4); /*Ano*/
   DECLARE @V_MESFIM          VARCHAR(2); /*Ms*/
   DECLARE @V_ANOFIM          VARCHAR(4); /*Ano*/

   /* VARIAVEIS DO CURSOR C_ALIMENTA */
   DECLARE @C_PRODUCAO_CDFILIAL        VARCHAR(4);
   DECLARE @C_PRODUCAO_MES             VARCHAR(2);
   DECLARE @C_PRODUCAO_ANO             VARCHAR(4);
   DECLARE @C_PRODUCAO_CDARVPROD       VARCHAR(100);
   DECLARE @C_PRODUCAO_SGUNIDADE       VARCHAR(3);
   DECLARE @C_PRODUCAO_TIPO            VARCHAR(10);
   DECLARE @C_PRODUCAO_SITUACAO        VARCHAR(20);
   DECLARE @C_PRODUCAO_QTAPRODPREOP    DOUBLE PRECISION;
   DECLARE @C_PRODUCAO_QTPRACABADO     DOUBLE PRECISION;
   DECLARE @C_PRODUCAO_PRODPERDA       VARCHAR(100);
   DECLARE @C_PRODUCAO_QTPERDAOP       DOUBLE PRECISION;
   DECLARE @C_PRODUCAO_VRCUSTOPREV     DOUBLE PRECISION;
   DECLARE @C_PRODUCAO_VRCUSTOREAL     DOUBLE PRECISION;
   DECLARE @C_PRODUCAO_VRCUSTOGERAL    DOUBLE PRECISION;
   DECLARE @C_PRODUCAO_MATERIAPRIMA    VARCHAR(100);
   DECLARE @C_PRODUCAO_CENTROCUSTO     VARCHAR(60);
   DECLARE @C_PRODUCAO_SETOR           VARCHAR(60);
   DECLARE @C_PRODUCAO_UNIDADEMP       VARCHAR(3);
   DECLARE @C_PRODUCAO_TIPOMOVI        VARCHAR(10);
   DECLARE @C_PRODUCAO_CUSTOITEM       DOUBLE PRECISION;
   
BEGIN

  SET @V_MESINI = (CONVERT(VARCHAR, MONTH(@P_DTINI))); 
  SET @V_MESINI = RIGHT(REPLICATE('0',2)+ @V_MESINI, 2);
  SET @V_ANOINI = (CONVERT(VARCHAR, YEAR (@P_DTINI)));

  SET @V_MESINI = (CONVERT(VARCHAR, MONTH(@P_DTINI)));
  SET @V_MESFIM = RIGHT(REPLICATE('0',2)+ @V_MESFIM, 2);
  SET @V_ANOFIM = (CONVERT(VARCHAR, YEAR (@P_DTINI)));

  BEGIN TRAN;
     DELETE FROM ETL_PRODUCAO
           WHERE ((ANOPRODU BETWEEN @V_ANOINI AND @V_ANOFIM)
              OR (MESPRODU BETWEEN @V_MESINI AND @V_MESFIM));
  COMMIT TRAN;

  DECLARE C_PRODUCAO CURSOR FOR SELECT OP.CDFILIAL,
                                       (CONVERT(VARCHAR, MONTH(OP.DTHRINIPREOP))) MES,
                                       (CONVERT(VARCHAR, YEAR (OP.DTHRINIPREOP))) ANO,
                                       PR.CDARVPROD,
                                       PR.SGUNIDADE,
                                       CASE OP.IDORDPRODCON WHEN 'N'THEN 'Normal' ELSE 'Consolidada' END TIPO,
                                       CASE OP.IDSITORDPROD WHEN '1' THEN 'Pendente' ELSE (CASE OP.IDSITORDPROD WHEN '2' THEN 'Em Andamento' ELSE (CASE OP.IDSITORDPROD WHEN '3' THEN 'Cancelada' ELSE ('Reprovada') END) END) END SITUACAO,
                                       ISNULL(OP.QTAPRODPREOP,0) QTAPRODPREOP,
                                       ISNULL(PA.QTPRACABADO,0) QTPRACABADO,
                                      (SELECT PX.CDARVPROD FROM PRODUTO PX WHERE PX.CDPRODUTO = PE.CDPRODUTO) PRODPERDA,
                                       ISNULL(PE.QTPERDAOP,0) QTPERDAOP,
                                       CASE OP.IDSITORDPROD WHEN '1' THEN 0 ELSE (CASE OP.IDSITORDPROD WHEN '2' THEN ROUND(TEKNISA.CUSTO_PRODUTO(OP.CDFILIAL, OP.CDPRODUTO, PAR.NRTPCUSTOOP, GETDATE(), GETDATE()), 5) ELSE (CASE OP.IDSITORDPROD WHEN '3' THEN ROUND(TEKNISA.CUSTO_PRODUTO(OP.CDFILIAL, OP.CDPRODUTO, PAR.NRTPCUSTOOP, GETDATE(), GETDATE()), 5) ELSE (0) END) END) END VRCUSTOPREV,
                                       CASE OP.IDSITORDPROD WHEN '3' THEN PA.VRPRACABADO ELSE 0 END VRCUSTOREAL,
                                       ISNULL((OP.QTAPRODPREOP*PA.VRPRACABADO),0) VRCUSTOGERAL,
                                       (SELECT CDARVPROD+' - '+NMPRODUTO FROM PRODUTO WHERE CDPRODUTO = IT.CDPRODUTO) AS MATERIAPRIMA,
                                       (SELECT NMCENTCUST FROM CENTCUST WHERE CDCENTCUST = RE.CDCENTCUST) CENTROCUSTO,
                                       (SELECT NMSETOR FROM SETOR WHERE CDSETOR = RE.CDSETOR) AS SETOR,
                                       (SELECT SGUNIDADE FROM PRODUTO WHERE CDPRODUTO = IT.CDPRODUTO) AS UNIDADEMP,
                                       CASE LT.IDTPRETEST WHEN 'R' THEN 'Retirada' ELSE 'Devoluo'END TIPOMOVI,
                                       IT.QTPRERETPROD QTPREVISTA, LA.QTLANCTOEST, ROUND(LA.VRUNILANCTO,2) AS CUSTOITEM
                                  FROM ORDPRODUCAO OP LEFT JOIN PRODACABADO PA ON (PA.NRORDPROD     = OP.NRORDPROD 
                                                                                   AND PA.NRSUBORDPROD  = OP.NRSUBORDPROD
                                                                                   AND PA.CDPRODUTO     = OP.CDPRODUTO
                                                                                   AND PA.CDFILIAL      = OP.CDFILIAL )
                                                       LEFT JOIN PERDAOP PE ON    (PE.CDFILIAL         = OP.CDFILIAL 
                                                                                   AND PE.NRORDPROD    = OP.NRORDPROD
                                                                                   AND PE.NRSUBORDPROD = OP.NRSUBORDPROD)
                                                       LEFT JOIN RETIRADAESTO RE ON    (RE.CDFILIAL         = OP.CDFILIAL 
                                                                                        AND RE.NRORDPROD    = OP.NRORDPROD
                                                                                        AND RE.NRSUBORDPROD = OP.NRSUBORDPROD),

                                       PRODUTO PR,
                                       PARAMAN PAR,
                                       ITEMRETIEST IT,
                                       ITLANCTOEST LA,
                                       LOTITRETIEST LT
                                 WHERE OP.CDPRODUTO        = PR.CDPRODUTO
                                   AND OP.IDORDPRODCON     <> 'O'
                                   
                                   
                                   AND PE.CDFILIAL      = OP.CDFILIAL
                                   AND PE.NRORDPROD     = OP.NRORDPROD
                                   AND PE.NRSUBORDPROD  = OP.NRSUBORDPROD
                                   AND PAR.CDFILIAL     = OP.CDFILIAL
                                   AND RE.NRORDPROD     = OP.NRORDPROD
                                   AND RE.NRSUBORDPROD  = OP.NRSUBORDPROD
                                   AND RE.CDFILIAL      = OP.CDFILIAL
                                   AND RE.CDFILIAL      = IT.CDFILIAL
                                   AND RE.NRRETEST      = IT.NRRETEST
                                   AND LA.CDFILIAL      = RE.CDFILIAL
                                   AND LA.NRLANCESTQ    = RE.NRLANCESTQ
                                   AND LT.NRRETEST      = IT.NRRETEST
                                   AND LT.CDFILIAL      = IT.CDFILIAL
                                   AND ((OP.DTHRINIPREOP > @P_DTINI) AND (OP.DTHRINIPREOP < @P_DTFINA))

   OPEN C_PRODUCAO;
   FETCH NEXT FROM C_PRODUCAO INTO @C_PRODUCAO_CDFILIAL,
                                   @C_PRODUCAO_MES,
                                   @C_PRODUCAO_ANO,
                                   @C_PRODUCAO_CDARVPROD,
                                   @C_PRODUCAO_SGUNIDADE,
                                   @C_PRODUCAO_TIPO,
                                   @C_PRODUCAO_SITUACAO,
                                   @C_PRODUCAO_QTAPRODPREOP,
                                   @C_PRODUCAO_QTPRACABADO,
                                   @C_PRODUCAO_PRODPERDA,
                                   @C_PRODUCAO_QTPERDAOP,
                                   @C_PRODUCAO_VRCUSTOPREV,
                                   @C_PRODUCAO_VRCUSTOREAL,
                                   @C_PRODUCAO_VRCUSTOGERAL,
                                   @C_PRODUCAO_MATERIAPRIMA,
                                   @C_PRODUCAO_CENTROCUSTO,
                                   @C_PRODUCAO_SETOR,
                                   @C_PRODUCAO_UNIDADEMP,
                                   @C_PRODUCAO_TIPOMOVI,
                                   @C_PRODUCAO_CUSTOITEM;
   WHILE (@@FETCH_STATUS = 0)
   BEGIN
       BEGIN TRAN;

       INSERT INTO ETL_PRODUCAO
       (CDFILIAL, CDANOBI, CDMESBI, NMPRODUTOPROD, SGUNIDADEPROD, IDTIPOOP,
        NMSITUACAOOP, QTPREVPROD, QTREALPROD, NMPRODPERDA, QTPERDAPROD,
        VRCUSTOPREV, VRCUSTOREAL, VRCUSTOGERAL, NMMATPRIMA, NMCENTCUSTMO,
        NMSETORMP, SGUNIDADEMP, NMTPRETIRADA, VRCUSTOITEM)
       VALUES
       (@C_PRODUCAO_CDFILIAL,     @C_PRODUCAO_MES,         @C_PRODUCAO_ANO,          @C_PRODUCAO_CDARVPROD,   @C_PRODUCAO_SGUNIDADE,
        @C_PRODUCAO_TIPO,         @C_PRODUCAO_SITUACAO,    @C_PRODUCAO_QTAPRODPREOP, @C_PRODUCAO_QTPRACABADO,
        @C_PRODUCAO_PRODPERDA,    @C_PRODUCAO_QTPERDAOP,   @C_PRODUCAO_VRCUSTOPREV,  @C_PRODUCAO_VRCUSTOREAL, @C_PRODUCAO_VRCUSTOGERAL,
        @C_PRODUCAO_MATERIAPRIMA, @C_PRODUCAO_CENTROCUSTO, @C_PRODUCAO_SETOR,        @C_PRODUCAO_UNIDADEMP,   @C_PRODUCAO_TIPOMOVI, @C_PRODUCAO_CUSTOITEM);

      COMMIT TRAN;
      FETCH NEXT FROM C_PRODUCAO INTO @C_PRODUCAO_CDFILIAL,
                                      @C_PRODUCAO_MES,
                                      @C_PRODUCAO_ANO,
                                      @C_PRODUCAO_CDARVPROD,
                                      @C_PRODUCAO_SGUNIDADE,
                                      @C_PRODUCAO_TIPO,
                                      @C_PRODUCAO_SITUACAO,
                                      @C_PRODUCAO_QTAPRODPREOP,
                                      @C_PRODUCAO_QTPRACABADO,
                                      @C_PRODUCAO_PRODPERDA,
                                      @C_PRODUCAO_QTPERDAOP,
                                      @C_PRODUCAO_VRCUSTOPREV,
                                      @C_PRODUCAO_VRCUSTOREAL,
                                      @C_PRODUCAO_VRCUSTOGERAL,
                                      @C_PRODUCAO_MATERIAPRIMA,
                                      @C_PRODUCAO_CENTROCUSTO,
                                      @C_PRODUCAO_SETOR,
                                      @C_PRODUCAO_UNIDADEMP,
                                      @C_PRODUCAO_TIPOMOVI,
                                      @C_PRODUCAO_CUSTOITEM;
   END;
   CLOSE C_PRODUCAO;
   DEALLOCATE C_PRODUCAO;
END;
GO
-- VERSAO --
EXEC ATUVEROBJ 'ETL_ORDPRODUCAO','5.30.000';
GO